function [T,F,G]=cor2tfg(plant,SYS_Kr,tol)
%
%==========================================================
% Observer-Based Realization of a given controller
%==========================================================
%
% [T,F,G] = COR2TFG(PLANT,SYS_K) 
% Given a nk order continuous-time controller SYS_K defined 
% by a state space realization (Ak, Bk, Ck, Dk) associated 
% with state variable Xk,
% given a n order given continous-time plant PLANT defined by 
% a state space realization (A,B,C,D) associated with state 
% variable X,
% COR2TGF compute the matrix T (nk x n) of the linear combination 
% Z=TX of the plant states X which are estimated by the controller 
% state Xk; that is Xk=Z_hat.
% F and G are the dynamic matrix and the input matrix of the 
% Luenberger observer associated with the controller, that is:
% let u and y be the input and the output of the plant, the 
% controller SYS_K can be parametrized in the following way:
%
%      X_k = F X_k + G (y-Du) + T B u
%        u = Ck X_k + Dk y
%
% Remarks: * SYS_K, PLANT are defined as SS objects,
%          * feedback between PLANT and SYS_K is positive,
%          * a real solution may not exit,
% 
% This function plots the map of closed-loop eigenvalues (red x)
% and PLANT open-loop eigenvalues (blue +) in the complex plane. 
% Then, the user can choose, in a interactive procedure, the 
% nk closed-loop eigenvalues which must be assigned to the Luenberger
% observer dynamics (spec(F); marked with red o).
%
% Uncontrollable eigenvalues (blue o) are automatically not assigned 
% to spec(F).
% Unobservable eigenvalues (red o) are automatically assigned to spec(F).
% (the controller SYS_K is assumed to be minimal).
% Auto-conjugate eigenvalues are assigned together.
%
%
% [T,F,G] = COR2TFG(PLANT,SYS_K,TOL) allows a tolerance TOL
% (default: 10^-6) to be taken into account in the unobservable
% and uncontrollable subspaces computation.
%
% Reference: D. Alazard and P. Apkarian "Exact observer based structures
% for arbitrary compensators" International Journal of Robust and 
% Non-Linear Control, 1999, n.9, pp 101-118 
%
% See also: COR2TFGA, COR2OBR, COR2OBRA, OBR2COR

% D. Alazard 11/06
% revised: 11/10 to handle SS objects
% Copyright (c) 1993-2006 SUPAERO, All Rights Reserved.

% Closed-loop matrix computation:

if nargin==2, tol=10^-6;end;
[ac,bc,cc,dc]=ssdata(SYS_Kr);
[A,B2,C2,D22]=ssdata(plant);
n=size(A,1);nk=size(ac,1);ntot=n+nk;
[acd,bcd,ccd,dcd]=feedback(ac,bc,cc,dc,[],[],[],-D22);
[abf,bbf,cbf,dbf]=feedback(A,B2,C2,0*D22,acd,bcd,ccd,dcd,1);

% ===================================================
% Closed-loop and open loop eigen values map 
% ===================================================
[V,D]=eig(abf);
lesvp=diag(D);
nbreal=find(imag(lesvp)==0);
if (mod(n,2)~=0),  % case n is odd
    if (length(nbreal)==0),  %  ===> no solution
        disp('No real solution !!');
        T=[];F=[];G=[];
        return
    end
end
figure
plot(real(lesvp),imag(lesvp),'rx','LineWidth',2);
hold on
vpG=eig(A);
plot(real(vpG),imag(vpG),'b+','LineWidth',2);
sgrid
zoom on

% ==================================================
% Interactive selection
% ==================================================
nbf=length(lesvp);
index1=[];
index2=[];
reste=[];
ii=1;
while ii<=nbf,
  valp=lesvp(ii);
  if imag(valp)==0;
      ir=ii;ii=ii+1;
  else
      ir=[ii ii+1];ii=ii+2;
  end
  if rank([A-valp*eye(n,n) B2],tol)<n,
      index1=[ir index1];
      if length(index1)>n, disp('No solution !!!');return;end;
      plot(real(lesvp(ir)),imag(lesvp(ir)),'bo','LineWidth',2);
  elseif rank([A'-valp*eye(n,n) C2'],tol)<n,
      index2=[index2 ir];
      if length(index2)>n, disp('No solution !!!');return;end;
      plot(real(lesvp(ir)),imag(lesvp(ir)),'ro','LineWidth',2);
  else
      reste=[reste ir];
  end
end
index1s=index1;
index2s=index2;
ii=1;
while ii<=length(reste),
   valp=lesvp(reste(ii));
   if imag(valp)==0;
      ir=reste(ii);ii=ii+1;
   else
      ir=[reste(ii) reste(ii+1)];ii=ii+2;
   end
   ok=0;
   while ~ok
      text=['Do you want to assign: ',...
             num2str(valp),' to Luenberger observer dynamics: spec(F) ?'];
      choix=menu(text,'no','yes','restart procedure');
      switch choix,
      case 1, 
         if length(index1)==n,
             disp('Selection is completed');
             ok=1;index2=[ir reste(ii:length(reste)) index2];ii=length(reste)+1;
         elseif length(index1)+length(ir)>n
             disp('Complex eigenvalue must be now assigned to spec(F) !!');
         else
             ok=1;index1=[index1 ir];
             plot(real(lesvp(ir)),imag(lesvp(ir)),'bo','LineWidth',2);
             if length(index1)==n,
                 disp('Selection is completed');
                 index2=[reste(ii:length(reste)) index2];ii=length(reste)+1;
             end
         end
      case 2,
         if length(index2)==nk,
             disp('Selection is completed');
             ok=1;index1=[ir reste(ii:length(reste)) index1];ii=length(reste)+1;
         elseif length(index2)+length(ir)>nk
             disp('Only a real eigenvalue can be now assigned to spec(F) !');
         else
             ok=1;index2=[ir index2];
             plot(real(lesvp(ir)),imag(lesvp(ir)),'ro','LineWidth',2);
             if length(index2)==nk,
                 disp('Selection is completed');
                 index1=[reste(ii:length(reste)) index1];ii=length(reste)+1;
             end
         end
      case 3,
         ok=1;index1=index1s;index2=index2s;ii=1;
      end
   end
end

index12=[index1 index2];
M=V(:,index12);
D=D(index12,index12);
[M,D]=cdf2rdf(M,D);

% ==================================================
% Oberver-based parameter computation
% ==================================================


T=real(M(n+1:nbf,1:n)*inv(M(1:n,1:n)));
F=acd-T*B2*ccd;
G=bcd-T*B2*dcd;

